// 002 u_c_array_of_record_of_doubles
// 30 jun 2007

// -- (C) John COLIBRI 2007
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_array_of_record_of_doubles;
  interface
    const k_granularity= 4;

    type t_double_record= Record
                            m_x, m_y, m_z: Double;
                          end; // t_double_record

         c_array_of_record_of_doubles=
             Class
               private
                 m_array_of_double_record: ARRAY of t_double_record;
                 m_count: Integer;

                 function get_doubles(p_index: Integer): t_double_record;
                 procedure set_doubles(p_index: Integer; p_double_record: t_double_record);

                 function get_x(p_index: Integer): Double;
                 procedure set_x(p_index: Integer; p_x: Double);

                 function get_y(p_index: Integer): Double;
                 procedure set_y(p_index: Integer; p_y: Double);

                 function get_z(p_index: Integer): Double;
                 procedure set_z(p_index: Integer; p_z: Double);
               public
                 Constructor create;
                 procedure add_doubles(p_double_record: t_double_record);

                 procedure set_length(p_length: Integer);

                 Destructor Destroy; Override;

                 Property Doubles[p_index: Integer] : t_double_record
                     Read get_doubles Write set_doubles; Default;
                 Property X[p_index: Integer] : Double
                     Read get_x Write set_x;
                 Property Y[p_index: Integer] : Double
                     Read get_y Write set_y;
                 Property Z[p_index: Integer] : Double
                     Read get_z Write set_z;

                 Property Count: Integer Read m_count Write m_count;
             end; // c_array_of_double

       function f_double_record(p_x, p_y, p_z: Double): t_double_record;

  implementation
    Uses SysUtils;

    function f_double_record(p_x, p_y, p_z: Double): t_double_record;
      begin
        Result.m_x:= p_x;
        Result.m_y:= p_y;
        Result.m_z:= p_z;
      end; // f_double_record

    // -- c_array_of_record_of_doubles

    Constructor c_array_of_record_of_doubles.create;
      begin
        InHerited Create;
        SetLength(m_array_of_double_record, k_granularity);
      end; // create

    procedure c_array_of_record_of_doubles.add_doubles(p_double_record: t_double_record);
      begin
        if m_count>= Length(m_array_of_double_record)
          then SetLength(m_array_of_double_record, 2* Length(m_array_of_double_record));

        m_array_of_double_record[m_count]:= p_double_record;
        Inc(m_count);
      end; // add_double

    function c_array_of_record_of_doubles.get_doubles(p_index: Integer): t_double_record;
      begin
        if p_index>= Length(m_array_of_double_record)
          then Raise Exception.Create('p_index> m_count');

        Result:= m_array_of_double_record[p_index];
      end; // get_doubles

    procedure c_array_of_record_of_doubles.set_doubles(p_index: Integer; p_double_record: t_double_record);
      begin
        if p_index>= Length(m_array_of_double_record)
          then begin
              m_count:= p_index+ 1;
              SetLength(m_array_of_double_record, m_count);
            end;

        m_array_of_double_record[p_index]:= p_double_record;
      end; // set_doubles

    function c_array_of_record_of_doubles.get_x(p_index: Integer): Double;
      begin
        Result:= Doubles[p_index].m_x;
      end; // get_x

    procedure c_array_of_record_of_doubles.set_x(p_index: Integer; p_x: Double);
      begin
        if p_index>= Length(m_array_of_double_record)
          then begin
              m_count:= p_index+ 1;
              SetLength(m_array_of_double_record, m_count);
            end;

        m_array_of_double_record[p_index].m_x:= p_x;
      end; // set_x

    function c_array_of_record_of_doubles.get_y(p_index: Integer): Double;
      begin
        Result:= Doubles[p_index].m_y;
      end; // get_y

    procedure c_array_of_record_of_doubles.set_y(p_index: Integer; p_y: Double);
      begin
        if p_index>= Length(m_array_of_double_record)
          then begin
              m_count:= p_index+ 1;
              SetLength(m_array_of_double_record, m_count);
            end;

        m_array_of_double_record[p_index].m_y:= p_y;
      end; // set_y

    function c_array_of_record_of_doubles.get_z(p_index: Integer): Double;
      begin
        Result:= Doubles[p_index].m_z;
      end; // get_z

    procedure c_array_of_record_of_doubles.set_z(p_index: Integer; p_z: Double);
      begin
        if p_index>= Length(m_array_of_double_record)
          then begin
              m_count:= p_index+ 1;
              SetLength(m_array_of_double_record, m_count);
            end;

        m_array_of_double_record[p_index].m_z:= p_z;
      end; // set_z

    procedure c_array_of_record_of_doubles.set_length(p_length: Integer);
      begin
        m_array_of_double_record:= Nil;

        m_count:= p_length;
        SetLength(m_array_of_double_record, m_count);
      end; // set_length

    Destructor c_array_of_record_of_doubles.destroy;
      begin
        m_array_of_double_record:= Nil;

        Inherited;
      end; // destroy

end.
